@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # @Author  : ydf
 # @Time    : 2019/8/8 0008 12:12
+import json
 from function_scheduling_distributed_framework.publishers.base_publisher import AbstractPublisher
 from function_scheduling_distributed_framework.utils import RedisMixin
 
@@ -11,7 +12,6 @@ class RedisPublisher(AbstractPublisher, RedisMixin):
     """
 
     def concrete_realization_of_publish(self, msg):
-        # noinspection PyTypeChecker
         self.redis_db_frame.rpush(self._queue_name, msg)
 
     def clear(self):
@@ -27,4 +27,3 @@ class RedisPublisher(AbstractPublisher, RedisMixin):
         # self.redis_db7.connection_pool.disconnect()
         pass
 
-
